#!/usr/bin/env python

import numpy as np

__all__ = [
     'divisors',
     'is_prime',
     'prime_divisors',
     'generate_prime_numbers',
]

def divisors(n):
    """divisors(n) -> list of divisors of n
    >>> divisors(5)
    [1, 5]
    >>> divisors(2009)
    [1, 7, 41, 49, 287, 2009]
    >>>
    """
    l = []
    for i in range(1, n + 1):
        if n % i == 0:
            # i e' un divisore
            l.append(i)
    return l

def is_prime(n):
    """is_prime(n) -> True if n is prime, False otherwise
    >>> is_prime(1)
    False
    >>> is_prime(2)
    True
    >>> is_prime(29)
    True
    """
    if not isinstance(n, int):
        raise TypeError("invalid argument for is_prime: invalid type {}".format(type(n).__name__))
    if n <= 0:
        raise ValueError("invalid argument for is_prime: invalid value {}: not a positive integer".format(n))
    if n < 2:
        return False
    for i in range(2, n):
        if n % i == 0:
            # i e' un divisore di n -> n non e' primo
            return False
    return True

def prime_divisors(n):
    """prime_divisors(n) -> list of prime divisors of n
    >>> prime_divisors(2009)
    [7, 41]
    >>>
    """
    return filter(is_prime, divisors(n))

def generate_prime_numbers():
    """primes() -> generate all prime numbers
    >>> for i in generate_prime_numbers():
    ...     if i > 10:
    ...         break
    ...     print i
    ...
    2
    3
    5
    7
    >>>
    """
    i = 2
    while True:
         if is_prime(i):
             yield i
         i += 1


if __name__ == "__main__":
    import doctest
    doctest.testmod()
